<?php
/* ChiptuningReseller class */
if(strpos($_SERVER['SERVER_NAME'], 'ctr-wordpress.test') !== false) {
    define('TESTMODE', true);
    define('BASEURL', "http://ctr.test/api/v1/");
} else {
    define('TESTMODE', false);
    define('BASEURL', "https://app.chiptuningreseller.com/api/v1/");
}

class CHIPTUNINGRESELLER
{
	public $full_title = "CTRCONNECTOR";
    private $plugin_dir;
    private $catalogData;
    
    private $API_URL            = BASEURL . 'cardata/';
    private $API_SEARCH_URL     =  BASEURL . 'cardata/search';
    private $API_KEY            = '';

    /**
     * __construct
     * 
     * @access public
     *
     * @return mixed Value.
     */
    public function __construct()
    {
        $settings = $this->get_settings();
        $this->API_KEY    = 'Authorization: Bearer '. $settings['api_key'];
        $this->plugin_dir  = plugin_dir_path( __FILE__ );
        
        include_once $this->plugin_dir .'/base.ctr.php';


        new CTR_BASE();
        load_plugin_textdomain( 'chiptuningreseller', false, basename( dirname( __FILE__ ) ) . '/languages' ); 


        $this->init_hooks();
    }

    /**
    *
    * Initialisation hooks
    *
    ***/
    private function init_hooks() {
    	add_action('admin_menu', array($this, 'create_admin_menu'));
        add_action( 'wp_ajax_flush_cache', array($this, 'flush_cache'));
    }

    /**
     * add_admin_menu
     * 
     * @access public
     *
     * @return mixed Value.
     */
    public function create_admin_menu() {
        $notification = $this->check_api_connection() == false ? 'ChiptuningR <span class="awaiting-mod">error</span>' : 'ChiptuningReseller';
        
        $hook = add_menu_page(
            'ChiptuningReseller', 
            $notification,
            'manage_options', 
            '__chiptuningreseller', 
            array($this, 'settings_page'));
    }

    public function settings_page() {
       
        global $wpdb;
        $data = [];

        $catalogData = null;

    	if(isset($_POST['update_settings'])) {
            if(!empty($_POST['api_key'])) {
                $data['api_key'] = $_POST['api_key'];
            }

            $data['enable_cache'] = !empty($_POST['enable_cache']) ? 1 : 0;

            $data['include_css'] = !empty($_POST['include_css']) ? 1 : 0;

            if(!empty($_POST['mailto'])) {
                $data['mailto'] = $_POST['mailto'];
            }

            $data['show_price'] = !empty($_POST['show_price']) ? 1 : 0;

            if(!empty($_POST['start_url'])) {
                $data['start_url'] = $_POST['start_url'];
            }
            if(!empty($_POST['stage_text'])) {
                $data['stage_text'] = $_POST['stage_text'];
            }
            if(!empty($_POST['custom_css'])) {
                $data['custom_css'] = $_POST['custom_css'];
            }
            if(!empty($_POST['display_lang'])) {
                $data['display_lang'] = $_POST['display_lang'];
            }
            if(!empty($_POST['currency'])) {
                $data['currency'] = strlen($_POST['currency']) > 0 ? $_POST['currency'] : '€';
            }
            if(!empty($_POST['fixed_stage_prices'])) {
                $data['fixed_stage_prices'] = json_encode($_POST['fixed_stage_prices']);
            }
            if(!empty($_POST['show_stages'])) {
                $data['show_data']['stages'] = json_encode($_POST['show_stages']);
            }
            if(!empty($_POST['show_vehicles']) && !empty($_POST['show_stages'])) {
                $data['show_data']['vehicles'] = json_encode($_POST['show_vehicles']);
            }
            if(!empty($_POST['layout_settings'])) {
                $data['layout_settings'] = json_encode($_POST['layout_settings']);
            }
            if(!empty($_POST['reg_search'])) {
                $data['reg_search'] = json_encode($_POST['reg_search']);
            }
            if(isset($data['show_data'])) { 
                $data['show_data'] = json_encode($data['show_data']);
            }
            
            $response = $wpdb->update("{$wpdb->prefix}chiptuningreseller", $data, array('id'=> '1'));
            
            if ($response !== false)  {
                flush_rewrite_rules();
                $this->show_notice("success", __("Settings saved", "chiptuningreseller"));
            } else {
                $this->show_notice("error", __("Could not save settings", "chiptuningreseller") );
            }
        }
        
        $catalogData = $wpdb->get_row("SELECT * FROM {$wpdb->prefix}chiptuningreseller WHERE id=1" , ARRAY_A);
        $displayData = json_decode($catalogData['show_data'], true);
        
        //print_r($catalogData['show_stages'], true); exit;
        $catalogData['fixed_stage_prices'] = json_decode($catalogData['fixed_stage_prices'], true);
        $catalogData['show_stages'] = isset($displayData['stages']) ? json_decode($displayData['stages'], true) : '';
        $catalogData['show_vehicles'] = isset($displayData['vehicles']) ? json_decode($displayData['vehicles'], true) : '';
        $catalogData['layout_settings'] = json_decode($catalogData['layout_settings'], true);
        $catalogData['reg_search'] = json_decode($catalogData['reg_search'], true);
        $catalogData['connection_valid'] = $this->check_api_connection();
        $catalogData['languages'] = $this->get_languages();

        // Add the color picker css file       
        wp_enqueue_style( 'wp-color-picker' ); 
         
        // Include our custom jQuery file with WordPress Color Picker dependency
        wp_enqueue_script( 'custom-script-handle', plugins_url( '/assets/js/alpha-color-picker.js', __FILE__ ), array( 'wp-color-picker' ), false, true ); 

        $tab = ( ! empty( $_GET['tab'] ) ) ? esc_attr( $_GET['tab'] ) : 'first';
        $this->page_tabs( $tab );

        if ( $tab == 'first' ) {
            require_once(CHIPTUNINGRESELLER_PLUGIN_DIR . 'views/admin_settings.php');
        }
        if ( $tab == 'second' ) {
            require_once(CHIPTUNINGRESELLER_PLUGIN_DIR . 'views/admin_customize.php');
        }
        if ( $tab == 'third' ) {
            require_once(CHIPTUNINGRESELLER_PLUGIN_DIR . 'views/admin_fixed_prices.php');
        }
    }

    public function page_tabs( $current = 'first' ) {
        $tabs = array(
            'first'   => __( 'Main settings', 'plugin-textdomain' ), 
            'second'  => __( 'Layout & Customisation', 'plugin-textdomain' ),
            'third'  => __( 'Price settings', 'plugin-textdomain' )
        );
        $html = '<h2 class="nav-tab-wrapper">';
            foreach( $tabs as $tab => $name ){
                $class = ( $tab == $current ) ? 'nav-tab-active' : '';
                $html .= '<a class="nav-tab ' . $class . '" href="?page=__chiptuningreseller&tab=' . $tab . '">' . $name . '</a>';
            }
        $html .= '</h2>';
        echo $html;
    }

    /**
     * install the plugin's required assets into the db
     * 
     * @access public
     * @static
     *
     * @return mixed Value.
     */
    public static function plugin_activation() {
    	if ( version_compare( $GLOBALS['wp_version'], CHIPTUNINGRESELLER_MINIMUM_WP_VERSION, '<' ) ) {
			$message = '<strong>'.sprintf(esc_html_e( 'ChiptuningReseller %s requires WordPress %s or higher.' , 'chiptuningreseller'), CHIPTUNINGRESELLER_VERSION, CHIPTUNINGRESELLER_MINIMUM_WP_VERSION ).'</strong> '.sprintf(_e('Please <a href="%1$s">upgrade WordPress</a> to a current version.', 'chiptuningreseller'), 'https://codex.wordpress.org/Upgrading_WordPress');

			$this->bail_on_activation( $message );
		}

        global $wpdb;


        /** CREATION TABLE ChiptuningReseller */
        $wpdb->query("CREATE TABLE IF NOT EXISTS {$wpdb->prefix}chiptuningreseller 
                    (id INT  AUTO_INCREMENT PRIMARY KEY ,
                    company_name VARCHAR(255) DEFAULT NULL,
                    start_url TEXT DEFAULT NULL,
                    api_key TEXT DEFAULT NULL,
                    mailto VARCHAR(255) DEFAULT '',
                    enable_cache TINYINT(1) DEFAULT 1,
                    stage_text TEXT DEFAULT NULL,
                    display_lang VARCHAR(255) DEFAULT NULL,
                    include_css TINYINT(1) DEFAULT 0,
                    custom_css TEXT DEFAULT '',
                    show_price TINYINT DEFAULT '0',
                    currency VARCHAR(10) DEFAULT '€',
                    show_data TEXT DEFAULT NULL,
                    fixed_stage_prices TEXT DEFAULT NULL,
                    layout_settings TEXT DEFAULT NULL,
                    reg_search VARCHAR(255) DEFAULT NULL,
                    _time_creation DATETIME DEFAULT NULL);");

        $result = $wpdb->query("SELECT id FROM {$wpdb->prefix}chiptuningreseller WHERE id = 1");
        if($result !== 1) {
        /** Insert default line */
        $wpdb->query("INSERT INTO {$wpdb->prefix}chiptuningreseller (id, company_name, start_url, api_key, enable_cache, display_lang, include_css, show_price, show_data, layout_settings, custom_css, _time_creation) 
            VALUES (1, 'ChiptuningReseller', 'chip-tuning', '', 1, 'auto', 1, 0, '{\"stages\": [{\"stage_1\":\"1\",\"stage_2\":\"1\",\"stage_3\":\"1\",\"stage_4\":\"1\",\"stage_dsg\":\"1\",\"stage_extra\":\"1\"}]',
            '{\"show_graph\":\"1\",\"show_dyno\":\"1\",\"show_basic_table\":\"1\", \"show_selector\":\"0\",\"graph_colors\":{\"hp\":\"rgba(221,51,51,1)\",\"nm\":\"rgba(30,115,190,1)\"}}', 
            '/** The ChiptuningReseller plugin supports Twitter Bootstrap, if your theme has this already, you dont need to activate the Bootstrap plugin. **/
            /** Main CTR container **/
            .chiptuningreseller-container {}
            /** Theme sidebar space (if active) **/
            .ctrSidebar {}
            /** Theme footer **/
            .ctrFooter {}
            /** Breadcrumbs **/
            .breadcrumbs {}
            /** Horizontal divider (shows between every 4 blocks) **/
            .horDivider {width: 100%; height: 1px; background-color: #cccccc;}
            /** The main CTR row that contains all other stuff **/
            .ctrMainRow {}
            /** CTR Blocks row **/
            .ctrRow {}
            /** Title above the Brands, Series, Builds and Engines **/
            .ctrTitle {}
            /** Title above the stage overview **/
            .product-title {}
            /** A block with brand, serie, build or engine **/
            .ctrBlock {}
            /** Specific brand block **/
            .brandBlock {}
            /** Specific serie block **/
            .serieBlock {}
            /** Specific build block **/
            .buildBlock ()
            /** Specific engine block **/
            .engineBlock {}
            /** Div with Brand Image **/
            .brandImage {}', '".date('d-m-Y H:i:s')."')");
        }
  
        /**** Create the file upload directory ****/
        $uploadDir =  wp_upload_dir();
        $fileDir = $uploadDir['basedir'] . '/ctr';

        if (!is_dir($fileDir)) {
            mkdir($fileDir, 0755);
        }

        /*** Copy default images ***/
        $mediaDir = plugin_dir_path( __FILE__ ) . '/assets/media'; 
        self::recurse_copy($mediaDir, $fileDir . '/assets');

        /** Make sure all rewriting for the new plugin works as it should **/
        flush_rewrite_rules();
    }


    /**
     * uninstall the plugin's db
     * 
     * @access public
     * @static
     *
     * @return mixed Value.
     */
    public static function plugin_deactivation()  {
        global $wpdb;

        $wpdb->query("DROP TABLE IF EXISTS {$wpdb->prefix}chiptuningreseller;");
    }
 
    /**
     * uninstall
     * 
     * @access public
     * @static
     *
     * @return mixed Value.
     */
    public static function plugin_deinstallation() {
        global $wpdb;

        $wpdb->query("DROP TABLE IF EXISTS {$wpdb->prefix}chiptuningreseller;");
    }


    /**
    * Show a notice of $type with $message
    * @param string $type
    * @param $message
    */
    public function show_notice( $type = "success", $message ) {
        ?>
            <div class="notice notice-<?php echo $type; ?> is-dismissible">
                <p><?php echo $message; ?></p>
            </div>
        <?php
    }


    /****
    *
    * Check if the API connection is valid
    *
    *
    ***/
    public function check_api_connection() {
        global $wpdb;
        $catalogData = null;
        $catalogData = $wpdb->get_row("SELECT * FROM {$wpdb->prefix}chiptuningreseller WHERE id=1", ARRAY_A);
        $url = BASEURL . 'check';
        $response = wp_remote_get(
                $url, array(
                    'body' => '',
                    'timeout' => 30,
                    'headers' => [
                    'Authorization' => 'Bearer ' . $catalogData['api_key']
                ]
            )
        );
        if (is_wp_error( $response )) {
           $error_message = $response->get_error_message();
           //echo "Something went wrong: $error_message";
        } else {
           $responseData = json_decode($response['body'], true);
           if($responseData !== 'error' && (isset($responseData['currentUser']))) {
                return $responseData['currentUser'];
           }
           return false;
        }
    }


    /**
     * Check if data already exists in cache and if so, return it.
     */
    function get_cache($key, $group = 'ctr')
    {
        return get_transient( $group . '_' . $key );
    }

    /**
     * Set cached data
     */
    function set_cache($key, $data, $group = 'ctr', $expire = 0)
    {
        //return wp_cache_set($key, $data, $group, $expire);
        return set_transient( $group . '_' . $key, $data, $expire );
    }

    /***
    *
    * Get a list with vehicle types
    *
    ***/
    public function get_types() {
        $cached = $this->get_cache('types');
        if(false !== $cached && $this->get_settings()['enable_cache'] == 1) {
            return $cached;
        }

        global $wpdb;
        $catalogData = null;
        $catalogData = $wpdb->get_row("SELECT * FROM {$wpdb->prefix}chiptuningreseller WHERE id=1", ARRAY_A);
        $url = $url = $this->API_URL . 'types';
        $response = wp_remote_get(
                $url, array(
                    'body' => '',
                    'timeout' => 30,
                    'headers' => [
                    'Authorization' => 'Bearer ' . $catalogData['api_key']
                ]
            )
        );
        if (is_wp_error( $response )) {
           $error_message = $response->get_error_message();
           //echo "Something went wrong: $error_message";
        } else {
           $responseData = json_decode($response['body'], true);
           $this->set_cache('types', $responseData);
           return $responseData;
        }
    }

    /***
    *
    * Get an array with settings information to use on front-end
    *
    ***/
    public function get_settings() {
        global $wpdb;
        $catalogData = null;
        $table_name = $wpdb->prefix.'chiptuningreseller';
        if($wpdb->get_var("SHOW TABLES LIKE '$table_name'") != $table_name) {
            return null;
        }
        
        $catalogData = $wpdb->get_row("SELECT api_key, enable_cache, start_url, mailto, stage_text, display_lang, include_css, custom_css, show_price, currency, show_data, fixed_stage_prices, layout_settings, reg_search FROM {$wpdb->prefix}chiptuningreseller WHERE id=1", ARRAY_A);
        $displayData = json_decode($catalogData['show_data'], true);
        //print_r(json_decode($catalogData['show_data'], true)['stages']); exit;
        $catalogData['layout_settings'] = json_decode($catalogData['layout_settings'], true);
        $catalogData['fixed_stage_prices'] = json_decode($catalogData['fixed_stage_prices'], true);
        $catalogData['show_stages'] = isset($displayData['stages']) ? json_decode($displayData['stages'], true) : '';
        $catalogData['show_vehicles'] = isset($displayData['vehicles']) ? json_decode($displayData['vehicles'], true) : '';
        $catalogData['reg_search'] = json_decode($catalogData['reg_search'], true);
        $catalogData['custom_css'] = $this->minimizeCSS($catalogData['custom_css']);
        return $catalogData;
    }

    /**
    *
    * Set the vehicle type to be used in the API calls
    *
    ***/
    public function set_vehicle_type($type) {
        $type = $this->translate_vehicle_type($type);
        $this->vehicle_type = isset($type) ? $type : 'cars';
    }

    /***
    *
    * Translate the vehicle URL to the official slug URL
    * If the $lang param is set, the translated vehicle type will be returned according to the language
    *
    ***/
    public function translate_vehicle_type($search_type, $lang = null) {
        $search_type = strtolower($search_type);
        if($lang == 'auto') {
            $lang = null != get_locale() ? strtolower(get_locale()) : 'en_us';
        }

        $types['cars'] = [
            "nl_nl"    => "autos",
            "de_de"    => "autos",
            "en_us"    => "cars",
            "es_es"    => "autos",
            "fr_fr"    => "voitures",
            "it_it"    => "automobili",
            "pt_pt"    => "carros",
            "no_no"    => "biler",
            "nb_no"    => "bil",
            "da_dk"     => "biler"
        ];

        $types['bikesquad'] = [
            "nl_nl"    => "motoren-en-quads",
            "de_de"    => "fahrrader-und-quads",
            "en_us"    => "bikes-and-quads",
            "es_es"    => "bicicletas-y-quads",
            "fr_fr"    => "motos-et-quads",
            "it_it"    => "biciclette-e-quad",
            "pt_pt"    => "bicicletas-e-quads",
            "nb_no"    => "mc-og-atv",
            "no_no"    => "mc-og-atv",
            "da_dk"    => "motorer-og-quads"
        ];

        $types['motorboats'] = [
            "nl_nl"    => "boten",
            "de_de"    => "motorboote",
            "en_us"    => "boats",
            "es_es"    => "lanchas-a-motor",
            "fr_fr"    => "bateaux-a-moteur",
            "it_it"    => "motoscafi",
            "pt_pt"    => "lanchas",
            "no_no"    => "bater",
            "nb_no"    => "båt",
            "da_dk"    => "motorbade"
        ];

        $types['jet-skis'] = [
            "nl_nl"    => "jet-skis",
            "de_de"    => "jet-ski",
            "en_us"    => "jet-skis",
            "es_es"    => "jet-skis",
            "fr_fr"    => "jet-skis",
            "it_it"    => "moto-dacqua",
            "pt_pt"    => "jet-skis",
            "no_no"    => "vannscooter",
            "nb_no"    => "vannscooter",
            "da_dk"    => "jet-ski"
        ];

        $types['tractors'] = [
            "nl_nl"    => "tractoren",
            "de_de"    => "traktoren",
            "en_us"    => "tractors",
            "es_es"    => "tractores",
            "fr_fr"    => "tracteurs",
            "it_it"    => "trattori",
            "pt_pt"    => "tratores",
            "no_no"    => "traktorer",
            "nb_no"    => "traktor",
            "da_dk"    => "traktorer"
        ];

        $types['trucks'] = [
            "nl_nl"    => "trucks",
            "de_de"    => "lastwagen",
            "en_us"    => "trucks",
            "es_es"    => "camiones",
            "fr_fr"    => "camions",
            "it_it"    => "camion",
            "pt_pt"    => "camiones",
            "no_no"    => "lastebiler",
            "nb_no"    => "lastebil",
            "da_dk"    => "lastbiler"
        ];

        $types['snowmobiles'] = [
            "nl_nl"    => "sneeuwscooter",
            "de_de"    => "schneemobil",
            "en_us"    => "snowmobile",
            "es_es"    => "motonieve",
            "fr_fr"    => "motoneige",
            "it_it"    => "gatto-delle-nevi",
            "pt_pt"    => "moto-de-neve",
            "no_no"    => "snoscooter",
            "nb_no"    => "snoscooter",
            "da_dk"    => "snescootere"
        ];
        

        if($lang) {
            foreach($types as $key => $type) {
                if($search_type == $key) {
                    return $types[$key][$lang];
                }
            }
        } else {        
            foreach($types as $key => $type) {
                if(in_array($search_type, $type)) {
                    return $key;
                }
            }
        }

        return $search_type;
            
    }



    /***
    *
    * Translate the vehicle URL to the official slug URL
    * If the $lang param is set, the translated vehicle type will be returned according to the language
    *
    ***/
    public function translate_fuel_type($search_type, $lang = null) {
        $search_type = strtolower($search_type);
        if($lang == 'auto') {
            $lang = null != get_locale() ? strtolower(get_locale()) : 'en_us';
        }

        $strings = [
        "lbl_fuel_general"    => array(
            "nl_nl"               => "benzine",
            "de_de"               => "Benzin",
            "en_us"               => "gasoline",
            "es_es"               => "gasolina",
            "fr_fr"              => "essence",
            "it_it"               => "benzina",
            "pt_pt"               => "gasolina",
            "ru_ru"               => "бензин",
            "no_no"               => "bensin",
            "nb_no"             => "bensin"
        ),
        "lbl_fuel_gasoline"    => array(
            "nl_nl"               => "benzine",
            "de_de"               => "Benzin",
            "en_us"               => "gasoline",
            "es_es"               => "gasolina",
            "fr_fr"              => "essence",
            "it_it"               => "benzina",
            "pt_pt"               => "gasolina",
            "ru_ru"               => "бензин",
            "no_no"               => "bensin",
            "nb_no"             => "bensin"
        ),
        "lbl_fuel_diesel"    => array(
            "nl_nl"               => "diesel",
            "de_de"               => "Diesel",
            "en_us"               => "diesel",
            "es_es"               => "diesel",
            "fr_fr"              => "diesel",
            "it_it"               => "diesel",
            "pt_pt"               => "diesel",
            "ru_ru"               => "дизель",
            "no_no"               => "diesel",
            "nb_no"             => "diesel"
        ),
        "lbl_fuel_hybrid"    => array(
            "nl_nl"               => "hybride",
            "de_de"               => "Hybrid",
            "en_us"               => "hybrid",
            "es_es"               => "híbrido",
            "fr_fr"              => "hybride",
            "it_it"               => "ibrido",
            "pt_pt"               => "híbrido",
            "ru_ru"               => "гибрид",
            "no_no"               => "diesel",
            "nb_no"             => "diesel"
        )
        ];

        if($lang) {
            
            foreach($strings as $key => $type) {
                if($search_type == $key) {
                    return $strings[$key][$lang];
                }
            }
        } else {        
            foreach($strings as $key => $type) {
                if(in_array($search_type, $type)) {
                    return $key;
                }
            }
        }
            
    }


    /***
    *
    * Get a list with vehicle types
    *
    ***/
    public function get_brands($typeSlug = 'cars', $brandId = null) {
        if (isset($typeSlug)) {
            $cached = $this->get_cache('brands_' . $typeSlug);
            if(false !== $cached && $this->get_settings()['enable_cache'] == 1) {
                return $cached;
            }

            global $wpdb;
            $catalogData = null;
            $catalogData = $wpdb->get_row("SELECT * FROM {$wpdb->prefix}chiptuningreseller WHERE id=1", ARRAY_A);
            $url = $this->API_URL . 'brands?vehicle_type=' . $typeSlug;
            $response = wp_remote_get(
                    $url, array(
                        'body' => '',
                        'timeout' => 30,
                        'headers' => [
                        'Authorization' => 'Bearer ' . $catalogData['api_key']
                    ]
                )
            );
            if (is_wp_error( $response )) {
               $error_message = $response->get_error_message();
               //echo "Something went wrong: $error_message";
            } else {
                $responseData = json_decode($response['body'], true);
               if(isset($brandId)) {
                    foreach($responseData as $brand) {
                        if($brand['id'] == $brandId) {
                            return $brand;
                        };
                    }
                }
               $this->set_cache('brands_' . $typeSlug, $responseData, 'ctr', 180); // Save our brands for at least 30 days
               return $responseData;
            }
        }
    }

    /***
    *
    * Get a list with vehicle series
    *
    ***/
    public function get_series($brandId = NULL, $brandSlug = NULL) {
        if (isset($brandId) || isset($brandSlug)) {
            $cacheCode = $this->vehicle_type . '_series_' . base64_encode($brandId . '_' . $brandSlug);
            $serieCache = $this->get_cache($cacheCode);
            if(false !== $serieCache && $this->get_settings()['enable_cache'] == 1) {
                return $serieCache;
            }

            global $wpdb;
            $catalogData = null;
            $catalogData = $wpdb->get_row("SELECT * FROM {$wpdb->prefix}chiptuningreseller WHERE id=1", ARRAY_A);
            $url = (isset($brandSlug)) ? $this->API_SEARCH_URL . '?type='. $this->vehicle_type .'&brand_slug=' . $brandSlug : $this->API_URL . 'series?brand_id=' . $brandId;

            $response = wp_remote_get(
                    $url, array(
                        'body' => '',
                        'timeout' => 30,
                        'headers' => [
                        'Authorization' => 'Bearer ' . $catalogData['api_key']
                    ]
                )
            );
            if (is_wp_error( $response )) {
               $error_message = $response->get_error_message();
               echo "Something went wrong: $error_message";
            } else {
               $responseData = json_decode($response['body'], true);
               $this->set_cache($cacheCode, $responseData, 'ctr', 180); // Save our brands for at least 30 days
               return $responseData;
            }
        }
    }

    /***
    *
    * Get a list with vehicle builds
    *
    ***/
    public function get_builds($serieId = NULL, $brandSlug = NULL, $serieSlug = NULL) {
        if (isset($serieId) || (isset($brandSlug) && isset($serieSlug))) {
            $cacheCode = $this->vehicle_type . '_builds_' . base64_encode($brandSlug . '_' . $serieId .'_'. $serieSlug);
            $buildCache = $this->get_cache($cacheCode);
            if(false !== $buildCache && $this->get_settings()['enable_cache'] == 1) {
                return $buildCache;
            }

            global $wpdb;
            $catalogData = null;
            $catalogData = $wpdb->get_row("SELECT * FROM {$wpdb->prefix}chiptuningreseller WHERE id=1", ARRAY_A);
            $url = (isset($serieSlug)) ? $this->API_SEARCH_URL . '?type='. $this->vehicle_type .'&brand_slug=' . $brandSlug .'&serie_slug=' . $serieSlug : $this->API_URL . 'builds?serie_id=' . $serieId;
            $response = wp_remote_get(
                    $url, array(
                        'body' => '',
                        'timeout' => 30,
                        'headers' => [
                        'Authorization' => 'Bearer ' . $catalogData['api_key']
                    ]
                )
            );
            if (is_wp_error( $response )) {
               $error_message = $response->get_error_message();
               echo "Something went wrong: $error_message";
            } else {
               $responseData = json_decode($response['body'], true);
               $this->set_cache($cacheCode, $responseData, 'ctr', 180); // Save our builds for at least 30 days
               return $responseData;
            }
        }
    }

    /***
    *
    * Get a list with vehicle engines
    *
    ***/
    public function get_engines($buildId = NULL, $brandSlug = NULL, $serieSlug = NULL, $buildSlug = NULL, $searchTerm = NULL) {
        if (isset($buildId) || (isset($buildSlug)) && isset($brandSlug) && isset($serieSlug)) {
            $cacheCode = $this->vehicle_type . '_engines_' . base64_encode('_'. $brandSlug . '_' . $buildSlug . '' . $buildId);
            $engineCache = $this->get_cache($cacheCode);
            if(false !== $engineCache && $this->get_settings()['enable_cache'] == 1) {
                return $engineCache;
            }

            global $wpdb;
            $catalogData = null;
            $catalogData = $wpdb->get_row("SELECT * FROM {$wpdb->prefix}chiptuningreseller WHERE id=1", ARRAY_A);
            $url = (isset($buildSlug)) ? $this->API_SEARCH_URL . '?type='. $this->vehicle_type .'&brand_slug='. $brandSlug .'&serie_slug='. $serieSlug .'&build_slug=' . $buildSlug : $this->API_URL . 'engines?build_id=' . $buildId;
            $response = wp_remote_get(
                    $url, array(
                        'body' => '',
                        'timeout' => 30,
                        'headers' => [
                        'Authorization' => 'Bearer ' . $catalogData['api_key']
                    ]
                )
            );
            if (is_wp_error( $response )) {
               $error_message = $response->get_error_message();
               echo "Something went wrong: $error_message";
            } else {
               $responseData = json_decode($response['body'], true);
               $this->set_cache($cacheCode, $responseData, 'ctr', 180); // Save our builds for at least 30 days
               return $responseData;
            }
        }
    }
    
    /***
    *
    * Get a list with vehicle stages
    *
    ***/
    public function get_stages($engineId = NULL,  $brandSlug = NULL, $serieSlug = NULL, $buildSlug = NULL, $engineSlug = NULL){
        if (isset($engineId) || (isset($buildSlug) && isset($brandSlug) && isset($serieSlug) && isset($engineSlug))) {
            $cache_code = $this->vehicle_type . '_stages_' . base64_encode($buildSlug . '_' . $engineId);
            $stageCache = $this->get_cache($cache_code);
            if(false !== $stageCache) {
                return $stageCache;
            }
            global $wpdb;
            $catalogData = null;
            $catalogData = $wpdb->get_row("SELECT * FROM {$wpdb->prefix}chiptuningreseller WHERE id=1", ARRAY_A);
            $url = (isset($engineSlug)) ? $this->API_SEARCH_URL . '?type='. $this->vehicle_type .'&brand_slug='. $brandSlug .'&serie_slug='. $serieSlug .'&build_slug=' . $buildSlug . '&engine_slug=' . $engineSlug : $this->API_URL . 'stages?engine_id=' . $engineId;
            $response = wp_remote_get(
                    $url, array(
                        'body' => '',
                        'timeout' => 30,
                        'headers' => [
                        'Authorization' => 'Bearer ' . $catalogData['api_key']
                    ]
                )
            );
            if (is_wp_error( $response )) {
               $error_message = $response->get_error_message();
               echo "Something went wrong: $error_message";
            } else {
               $responseData = json_decode($response['body'], true);
               $this->set_cache($cache_code, $responseData, 'ctr', 180); // Save our builds for at least 30 days
               return $responseData;
            }
        }
    }

    
    /***
    *
    * Get Media for a specific Model
    *
    *
    ***/
    public function get_model_media($type, $modelId, $size = 'thumb') {
        if (isset($modelId)) {
            $url = $this->API_URL . $type . '/' . $modelId . '/image';
            $uploadDir =  wp_upload_dir();
            $fileDir = $uploadDir['basedir'] . '/ctr/' . $type . '/';
            $filePath = $uploadDir['basedir'] . '/ctr/'. $type . '/' . $modelId . '_'. $size .'.jpg';
            $settings = $this->get_settings();
            $file = get_site_url() . '/wp-content/uploads/ctr/' . $type . '/' . $modelId . '_' . $size . '.jpg';
            

            if((file_exists($filePath)) && (filesize($filePath) > 100)) {
            //    Cache file is less than five minutes old. 
            //    Don't bother refreshing, just use the file as-is.
               $file = get_site_url() . '/wp-content/uploads/ctr/' . $type . '/' . $modelId . '_' . $size . '.jpg';
            } else {
                if (!is_dir($fileDir)) {
                    mkdir($fileDir);
                }
                $file = wp_remote_get(
                    $url, array(
                        'timeout' => 1,
                        'headers' => [
                            'Authorization' => 'Bearer ' . $settings['api_key'],
                            'Accept' => 'application/json'
                        ]
                    )
                );
                if((is_array($file)) && (strpos($file['body'], 'error') === false && strpos($file['body'], 'no_media') === false && strpos($file['body'], 'FileNotFoundException') === false)) {
                    file_put_contents($filePath, $file['body']);
                    $file = get_site_url() . '/wp-content/uploads/ctr/' . $type . '/' . $modelId . '_' . $size . '.jpg';
                } else {
                    $file = get_site_url() . '/wp-content/uploads/ctr/placeholder.png';
                }
            }
            return $file;
        }
    }

    /***
    *
    * Get media for a specific model
    *
    *
    ***/
    public function get_media($modelId = NULL, $modelType = 'images',  $modelSlug = NULL, $size = 'thumb') {
        if (isset($modelId) || isset($modelSlug)) {
            $url = (isset($modelSlug)) ? $this->API_SEARCH_URL . $modelType . '/' . $modelSlug . '/image' : $this->API_URL . 'media/' . $modelId .'?version=' . $size;
            $uploadDir =  wp_upload_dir();
            $fileDir = $uploadDir['basedir'] . '/ctr/'. $modelType . '/';
            $filePath = $uploadDir['basedir'] . '/ctr/'. $modelType . '/' . $modelId . '_'. $size .'.jpg';
            $settings = $this->get_settings();
            if((file_exists($filePath)) && (time() - filemtime($filePath) < 10 * 3600)) {
               // Cache file is less than 10 days old
               // Don't bother refreshing, just use the file as-is.
               $response = get_site_url() . '/wp-content/uploads/ctr/' . $modelType . '/' . $modelId . '_' . $size . '.jpg';
            } else {
                if (!is_dir($fileDir)) {
                    mkdir($fileDir);
                }
                $file = wp_remote_get(
                    $url, array(
                        'body' => '',
                        'timeout' => 30,
                        'headers' => [
                            'Authorization' => 'Bearer ' . $settings['api_key']
                        ]
                    )
                );
                if(strpos($file['body'], 'error') === false && strpos($file['body'], 'style') === false) {
                    file_put_contents($filePath, $file['body']);
                    $response = get_site_url() . '/wp-content/uploads/ctr/' . $modelType . '/' . $modelId . '_' . $size . '.jpg';
                }
            }
            return $response;
        }
    }

    /***
    *
    * Get vehicle information by registration search
    *
    ***/
    public function get_rdw_registration($registration) {
        global $wpdb;
        $settings = $this->get_settings();
        $url = $this->API_SEARCH_URL . '/registration/nl/' . $registration . '?show_extended=true';
        $response = wp_remote_get(
                $url, array(
                    'body' => '',
                    'timeout' => 30,
                    'headers' => [
                    'Authorization' => 'Bearer ' . $settings['api_key']
                ]
            )
        );
        
        if (is_wp_error( $response )) {
           $error_message = $response->get_error_message();
           echo "Something went wrong: $error_message";
        } else {
            $response = wp_remote_get(
                $url, array(
                    'body' => '',
                    'timeout' => 30,
                    'headers' => [
                    'Authorization' => 'Bearer ' . $settings['api_key']
                    ]
                )
            );
            $data = [];
            $responseData = json_decode($response['body'], true);
            /** We need to fix the translation for the field 'type' in brands **/
            foreach($responseData as $k => &$v) {
                if($v['brand']) {
                    foreach($v['brand'] as $brK => &$brV) {
                        if($brK == 'type') {
                            $brV = $this->translate_vehicle_type($brV, $settings['display_lang']);
                        }
                    }
                }
                if($v['engine']) {
                    foreach($v['engine'] as $brK => &$brV) {
                        if($brK == 'fuel_type') {
                            $brV = $this->translate_fuel_type('lbl_fuel_' . $brV, $settings['display_lang']);
                        }
                    }
                }
                $correctedData[] = $v;
            }
           $responseData = json_encode($correctedData);
           echo $responseData;
           exit;
        }
    }

    /**
    *
    * Search a vehicle db wide
    *
    ***/
    public function search_for_term($searchTerm) {
        global $wpdb;
        $catalogData = null;
        $catalogData = $wpdb->get_row("SELECT * FROM {$wpdb->prefix}chiptuningreseller WHERE id=1", ARRAY_A);
        if(null == $searchTerm) {
            return null;
        }
        $url = $this->API_SEARCH_URL . '/byterm?search_term=' . $searchTerm;

        $response = wp_remote_get(
                $url, array(
                    'body' => '',
                    'timeout' => 30,
                    'headers' => [
                    'Authorization' => 'Bearer ' . $catalogData['api_key']
                ]
            )
        );
        if (is_wp_error( $response )) {
           $error_message = $response->get_error_message();
           echo "Something went wrong: $error_message";
        } else {
           $responseData = json_decode($response['body'], true);
           return $responseData;
        }
    }

    public function get_root_url() {
        global $wpdb;
        $catalogData = null;
        $catalogData = $wpdb->get_row("SELECT * FROM {$wpdb->prefix}chiptuningreseller WHERE id=1" , ARRAY_A );
        return isset($catalogData['start_url']) ? get_bloginfo('wpurl') . '/' . $catalogData['start_url'] : get_bloginfo('wpurl') . '/' . 'chip-tuning';
    }

    public function customer_login($data) {
        global $wpdb;
        $catalogData = null;
        $catalogData = $wpdb->get_row("SELECT * FROM {$wpdb->prefix}chiptuningreseller WHERE id=1" , ARRAY_A );

        $resData['email'] = sanitize_text_field($data['email']);
        $resData['password'] = sanitize_text_field($data['password']);

        $response = wp_remote_post(
            $this->res_login_url, array(
                'body' => $resData,
                'timeout' => 30,
                'headers' => [
                    'Authorization' => 'Bearer ' . $catalogData['api_key']
                ]
            )
        );
        if (is_wp_error($response)) {
           $error_message = $response->get_error_message();
           echo "Something went wrong: $error_message";
        } else {
           $responseData = json_decode($response['body'], true);
           return $responseData;
        }
    }

    /***
    *
    * Verify the payment against the ChiptuningReseller API
    *
    ***/
    public function verify_payment($transactionCode) {
        global $wpdb;
        $catalogData = null;
        $catalogData = $wpdb->get_row("SELECT * FROM {$wpdb->prefix}chiptuningreseller WHERE id=1" , ARRAY_A );

         $response = wp_remote_post(
            $this->res_verification_url, array(
                'body' => ['trx_code' => $transactionCode],
                'timeout' => 30,
                'headers' => [
                    'Authorization' => 'Bearer ' . $catalogData['api_key']
                ]
            )
        );
        if (is_wp_error( $response )) {
           $error_message = $response->get_error_message();
           echo "Something went wrong: $error_message";
        } else {
           $responseData = json_decode($response['body'], true);
           return $responseData;
        }
    }


    private static function recurse_copy($src, $dst) { 
        $dir = opendir($src); 
        @mkdir($dst); 
        while(false !== ( $file = readdir($dir)) ) { 
            if (( $file != '.' ) && ( $file != '..' )) { 
                if ( is_dir($src . '/' . $file) ) { 
                    recurse_copy($src . '/' . $file,$dst . '/' . $file); 
                } 
                else { 
                    copy($src . '/' . $file,$dst . '/' . $file); 
                } 
            } 
        } 
        closedir($dir); 
    }

    private function get_languages() {
        $options_lang = array(
            array( 'id' =>'auto' , 'name' => "Automatic" ),
            array( 'id' =>'en_us' , 'name' => "English (United States)" ),
            array( 'id' =>'nl_nl' , 'name' => "Nederlands" ),
            array( 'id' =>'fr_fr' , 'name' => "Français (France)" ),
            array( 'id' =>'es_es' , 'name' => "Español (España)" ),
            array( 'id' =>'ru_ru' , 'name' => "русский (Россия)" ),
            array( 'id' =>'pt_pt' , 'name' => "Português (Brasil)" ),
            array( 'id' =>'it_it' , 'name' => "Italiano (Italia)" ),
            array( 'id' =>'de_de' , 'name' => "Deutsch (Deutschland)" ),
            array( 'id' =>'da_dk' , 'name' => "Danish (Dansk)" ),
            array( 'id' =>'no_no' , 'name' => "Norsk (Nynorsk)" ),
            array( 'id' =>'nb_no' , 'name' => "Norsk (Bokmål )" )
        );
        return $options_lang;
    }

    public function flush_cache() {
        global $wpdb;
        
        if($wpdb->query("DELETE FROM {$wpdb->prefix}options WHERE option_name LIKE ('_transient_ctr_%');")) {
            echo 'All CTR cache items have been deleted!';
            wp_die();
        }
    }


    public function minimizeCSS($css){
        $css = preg_replace('/\/\*((?!\*\/).)*\*\//', '', $css); // negative look ahead
        $css = preg_replace('/\s{2,}/', ' ', $css);
        $css = preg_replace('/\s*([:;{}])\s*/', '$1', $css);
        $css = preg_replace('/;}/', '}', $css);
        return $css;
    }

    public function getTestmode() {
        return TESTMODE;
    }

    public function calculatePercentageIncrease($numberOne, $numberTwo) {
        return round((1 - $numberOne / $numberTwo) * 100);
    }
}