<?php

/**
 * Display functionalities
 *
 * This class display functions
 *
 * @since      3.0.0
 * @package    Ctr
 * @subpackage Ctr/includes
 * @author     M Bos <support@chiptuningreseller.com>
 */
require_once plugin_dir_path( dirname( __DIR__ ) ) . 'includes/class-ctr-db.php';
require_once plugin_dir_path( dirname( __DIR__ ) ) . 'includes/class-ctr-helpers.php';
require_once plugin_dir_path( dirname( __DIR__ ) ) . 'includes/class-ctr-seo.php';
require_once plugin_dir_path( dirname( __DIR__ ) ) . 'includes/class-ctr-rewrite.php';
require_once plugin_dir_path( dirname( __DIR__ ) ) . 'includes/class-ctr-media.php';
require_once plugin_dir_path( dirname( __DIR__ ) ) . 'includes/class-ctr-templator.php';
require_once plugin_dir_path( dirname( __DIR__ ) ) . 'includes/class-ctr-api-vehicle-types.php';
require_once plugin_dir_path( dirname( __DIR__ ) ) . 'includes/class-ctr-api-brands.php';
require_once plugin_dir_path( dirname( __DIR__ ) ) . 'includes/class-ctr-api-series.php';
require_once plugin_dir_path( dirname( __DIR__ ) ) . 'includes/class-ctr-api-builds.php';
require_once plugin_dir_path( dirname( __DIR__ ) ) . 'includes/class-ctr-api-engines.php';
require_once plugin_dir_path( dirname( __DIR__ ) ) . 'includes/class-ctr-api-stages.php';
require_once plugin_dir_path( dirname( __DIR__ ) ) . 'includes/class-ctr-api-search.php';
require_once plugin_dir_path( dirname( __DIR__ ) ) . 'public/includes/class-ctr-stagedata.php';

class Ctr_Display {
	private $seo;
	private $helpers;
	private $ctrTemplator;
	private $settings;
	
    public function __construct() {
		$this->helpers = new Ctr_Helpers();
		$this->rewrite_class = new CTR_Rewrite();
		$this->media = new CTR_Media();
		$this->seo = new CTR_SEO();

		$CtrDB = new Ctr_DB();
		$this->settings = $CtrDB->getSettings();
	}
	
	public function init() {
		$this->ctrTemplator = new CTR_Templator(plugin_dir_path( dirname( __DIR__ ) ) . 'public/templates/', plugin_dir_path( dirname( __DIR__ ) ) . 'public/cache/');
		$this->ctrTemplator->init();
	}

	public function renderFile($file, $injectData, $viewPath = null) {
		$this->ctrTemplator->renderFile($file, $injectData, $viewPath);
	}
    
    public function getSettings() {
		if(null != $this->settings) {
			return $this->settings;
		} else {
			$settings = [];
			$CtrDB = new Ctr_DB();
			$settings = $this->settings;
			return $settings;
		}
	}
	
	/** Set the query vars in our navigation data array */
	public function setNavigationData() {
		$this->navigation_data = $this->rewrite_class->get_query_vars();
	}

	/** Return the navigation data for usage */
	public function getNavigationData() {
		return $this->navigation_data;
	}

    public function handleFront() {
		$data = $this->rewrite_class->get_query_vars();
		$ROOT_URL = get_site_url() . '/'. $this->getSettings()['start_url'];
		$data['is_admin'] = (current_user_can('editor') || current_user_can('administrator')) ? true : false;
		$vehicle_type = isset($data['vehicle_type']) ? $this->helpers->translate_vehicle_type($data['vehicle_type']) : 'cars';
		if(isset($data['ctr_active'])) {
			if(isset($data['is_ajax'])) {
				$data['is_ajax'] = true;
			}

			/**
			 * Get media
			 * - First check if the get_media has been set, if so then switch the media type
			 * Then the get_model_media will get the data from the API and will then result in a 301 redirect
			 */
			if(isset($data['ctr_get_media'])) {
				$imgLocation = null;
				$media_size = isset($data['ctr_media_size']) ? $data['ctr_media_size'] : 'medium';
				while($imgLocation == null) {
					switch($data['ctr_media_type']) {
						case 'brand':
							$imgLocation = $this->media->get_model_media('brands', $data['ctr_media_id'], $media_size, $vehicle_type);
						break;
						case 'serie':
							$imgLocation = $this->media->get_model_media('series', $data['ctr_media_id'], $media_size, $vehicle_type);
						break;
						case 'build':
							$imgLocation = $this->media->get_model_media('builds', $data['ctr_media_id'], $media_size, $vehicle_type);
						break;
					}
				}
				if(null != $imgLocation) {
					header("Location: $imgLocation", true, 301);
				} else {
					return false;
				}
				exit;
			}
			$apiVehicleTypes = new Ctr_VehicleTypes();
			$data['vehicle_types'] = $apiVehicleTypes->getAllTypes();
			
			// Search API functionality, this functionality will only return JSON
			if(isset($data['ctr_search_term'])) {
				$apiSearch = new Ctr_Search();
				echo json_encode($apiSearch->searchAllData($data['ctr_search_term']));
				exit;
			}
			// Translate the vehicle type back to the en_us version to be able to use it later on
			$data['translated_vehicle_type'] = $vehicle_type;

			// Get Brands
			$apiBrands = new Ctr_Brands();
			$data['brands'] = $apiBrands->getBrands($data['vehicle_type']);
			$data['backUrl'] = $ROOT_URL;
			if(isset($data['vehicle_brand'])) {
				
                // Get Series
                $apiSeries = new Ctr_Series($data['vehicle_type']);
				$data['series'] = $apiSeries->getSeries(null, $data['vehicle_brand']);
				
				if(null == $data['series']) {
					$data['hasErrors'] = $this->ctrTemplator->returnFile('error', ['errorMessage' => 'An error occured, cannot continue. Maybe you entered an invalid URL?']);
					$data['errors'][] = 'No series found, please go back to the previous page and select a brand and then try again';
				} else {
					// Set brand name and image
					$data['brandId'] = $data['series']['brand']['id'];
					$data['brandName'] = $data['series']['brand']['name'];
					$data['brandSlug'] = $data['series']['brand']['slug'];
					$data['brandImage'] = $this->media->check_model_media('brands', $data['series']['brand']['id'], 'medium');
					$data['brandImageSrc'] = $data['brandImage'];
					if(isset($data['brandImage'])) { $data['brandImage'] = '<img src="' . $data['brandImage'] . '" />'; }
					unset($data['brands']); // clean up data
				}
				$data['backUrl'] = $ROOT_URL . '/' . $data['vehicle_type'];
			}
			if(isset($data['vehicle_serie'])) {
                // Get Series
                $apiBuilds = new Ctr_Builds($data['vehicle_type']);
				$data['builds'] = $apiBuilds->getBuilds(null, $data['vehicle_brand'], $data['vehicle_serie']);
				if(null == $data['builds']) {
					$data['hasErrors'] = $this->ctrTemplator->returnFile('error', ['errorMessage' => 'An error occured, cannot continue. Maybe you entered an invalid URL?']);
					$data['errors'][] = 'No series found, please go back to the previous page and select a brand and then try again';
				} else {
				// Set serie name
				$data['serieId'] = $data['builds']['serie']['id'];
				$data['serieName'] = $data['builds']['serie']['name'];
				$data['serieSlug'] = $data['builds']['serie']['slug'];
				
				// Get Serie Image
				$data['serieImage'] = $this->media->get_model_media('series', $data['serieId'], 'medium', $vehicle_type);
				$data['backUrl'] = $ROOT_URL . '/' . $data['vehicle_type'] . '/' . $data['vehicle_brand'];

				/* We only have ONE build, then redirect directly to this one */
				if((isset($data['builds']['count'])) && ($data['builds']['count'] == 1) && (!isset($data['vehicle_model']))) {
					//$data['vehicle_model'] = $data['builds'][0]['slug'];
					$actual_link = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? "https" : "http") . "://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]";
					$data['redirecting'] = true;
					echo '<meta http-equiv="refresh" content="0;URL='. $actual_link .'/' . $data['builds'][0]['slug'] .'" />';
				}
				unset($data['series']); // clean up data
			}
			}
			if(isset($data['vehicle_model'])) {
                // Get Engines
                $apiEngines = new Ctr_Engines($data['vehicle_type']);
				$data['engines'] = $apiEngines->getEngines(null, $data['vehicle_brand'], $data['vehicle_serie'], $data['vehicle_model']);
				if(null == $data['engines']) {
					$data['hasErrors'] = $this->ctrTemplator->returnFile('error', ['errorMessage' => 'An error occured, cannot continue. Maybe you entered an invalid URL?']);
					$data['errors'][] = 'No models found, please go back to the previous page and select a different build / model and then try again';
				} else {
					// Set build name
					$data['buildName'] = $data['engines']['build']['name'];
					$data['buildSlug'] = $data['engines']['build']['slug'];
					$data['backUrl'] = $stageData['url'] = $ROOT_URL . '/' . $data['vehicle_type'] . '/' . $data['vehicle_brand'] . '/' . $data['vehicle_serie'];
				}
				unset($data['builds']);
			}
			if(isset($data['vehicle_engine'])) {
				$engine_id = isset($data['vehicle_engine_id']) ? $data['vehicle_engine_id'] : null;
                // Get Engines
                $apiStages = new Ctr_Stages($data['vehicle_type']);
				$data['stages'] = $apiStages->getStages($engine_id, $data['vehicle_brand'], $data['vehicle_serie'], $data['vehicle_model'], $data['vehicle_engine']);
				$data['stageDisplayData'] = $this->prepareStageData($data['stages']);
				
				unset($data['engines']); // clean up data

				if(null == $data['stages']) {
					$data['hasErrors'] = $this->ctrTemplator->returnFile('error', ['errorMessage' => 'An error occured, cannot continue. Maybe you entered an invalid URL?']);
					$data['errors'][] = 'No data for this engine found, please go back to the previous page and select a different engine and then try again';
				} else {
					// Set engine name, slug and fuel type into the page title (for SEO)
					//$this->seo->set_title($data['brandName'] . ' ' . $data['serieName'] . ' ' . $data['buildName'] . ' ' . $data['stages']['vehicle']['engine']['name'] . ' ' . $data['stages']['vehicle']['engine']['fuel_type']);

					$data['backUrl'] = $stageData['url'] = $ROOT_URL . '/' . $data['vehicle_type'] . '/' . $data['vehicle_brand'] . '/' . $data['vehicle_serie'] . '/' . $data['vehicle_model'];
					
					/** Parse and prepare all stage data for sending to Blade template */
					$stageData = new Ctr_StageData($this->settings);
					
					$stageData->prepareData($data);
					$data = array_merge($data, $stageData->getData());
				}
			}
			
			// Set up SEO Titles
			$this->seo->set_seo_titles($data);
			return $data;
		}
	}

	public function handleAjax($data) {
		if(isset($data['vehicle_engine_id'])) {
			$apiStages = new Ctr_Stages($data['vehicle_type']);
			$stageData = $apiStages->getStages($data['vehicle_engine_id']);
			if(isset($stageData) && (is_array($stageData))) {
				$vehicle_type = $this->helpers->translate_vehicle_type($stageData['vehicle']['brand']['type']);
				$ROOT_URL = get_site_url() . '/'. $this->getSettings()['start_url'];
				
				$stageData['url'] = $ROOT_URL . '/' . $vehicle_type . '/' . $stageData['vehicle']['brand']['slug'] . '/' . $stageData['vehicle']['serie']['slug'] . '/' . $stageData['vehicle']['build']['slug'] . '/' . $stageData['engine']['slug'] . '/' . $stageData['engine']['id'];
			}
			wp_send_json($stageData);
			exit;
		}
		if(isset($data['vehicle_build_id'])) {
			$apiEngines = new Ctr_Engines($data['vehicle_build_id']);
			$engineData = $apiEngines->getEngines($data['vehicle_build_id']);
			wp_send_json($engineData);
			exit;
		}
		if(isset($data['vehicle_serie_id'])) {
			$apiBuilds = new Ctr_Builds($data['vehicle_type']);
			$buildData = $apiBuilds->getBuilds($data['vehicle_serie_id']);
			wp_send_json($buildData);
			exit;
		}
		elseif(isset($data['vehicle_brand_id'])) {
			$apiSeries = new Ctr_Series($data['vehicle_type']);
			$serieData = $apiSeries->getSeries($data['vehicle_brand_id']);
			wp_send_json($serieData);
			exit;
		}
		elseif(isset($data['vehicle_type'])) {
			wp_send_json($data['brands']);
			exit;
		}
	}

	/** Prepare all data for stage display template */
	public function prepareStageData($data) {
		$CtrDB = new Ctr_DB();
		$stageData = []; $inDevelopment = false; $i = 0;
		$stageDisplayData['in_development'] = false;
		foreach ($data['stages'] as $stage) {
			$showStage = ((isset($this->settings['show_stages'][$stage['name']])) && ($this->settings['show_stages'][$stage['name']] == '1')) ? true : false;
			if (strpos($stage['text_other'], 'in_development') !== false) {
				$inDevelopment = true; // IN DEVELOPMENT ON STAGE LEVEL
			}
			if ($showStage) {
				if (isset($stage['media'])) {
					$stageData[$i]['media'] = $stage['media'];
				}
				if (isset($stage['content'])) {
					$stageData[$i]['content'] = $stage['content'];
				}
				if (strpos($stage['name'], 'dsg') === false && strpos($stage['name'], 'extra') === false) {
					$stageData['hp_ori'] = round($stage['data']['hp_ori']);
					$stageData['nm_ori'] = round($stage['data']['nm_ori']);
					if (!$inDevelopment) {
						$stageData[$i]['hideStage'] = '';
						$stageData[$i]['name'] = $stage['name'];
						$stageData[$i]['display_name'] = $stage['display_name'];
						$stageData[$i]['hp_ori'] = round($stage['data']['hp_ori']);
						$stageData[$i]['hp_tuning'] = $stage['data']['hp_tuning'] > 0 ? $stage['data']['hp_tuning'] : '-';
						$stageData[$i]['hp_tuning_increase'] = $stage['data']['hp_tuning'] > 0 ? $stage['data']['hp_tuning'] -  $stage['data']['hp_ori'] : '-';
						$stageData[$i]['nm_ori'] = isset($stage['data']['nm_ori']) ? $stage['data']['nm_ori'] : '-';
						$stageData[$i]['nm_tuning'] = $stage['data']['nm_tuning'] > 0 ? $stage['data']['nm_tuning'] : '-';
						$stageData[$i]['nm_tuning_increase'] = $stage['data']['nm_tuning'] > 0 ? $stage['data']['nm_tuning'] -  $stage['data']['nm_ori'] : '-';
						$stageData[$i]['nm_tuning_increase_percentage'] = $this->helpers->calculatePercentageIncrease($stage['data']['nm_ori'], $stage['data']['nm_tuning']);
						$stageData[$i]['kw_ori'] = round($stage['data']['hp_ori'] * 0.745699872);
						$stageData[$i]['kw_tuning'] = $stage['data']['hp_tuning'] > 0 ? round($stage['data']['hp_tuning'] * 0.745699872) : '-'; 
						$stageData[$i]['kw_tuning'] = $stage['data']['hp_tuning'] > 0 ? round($stage['data']['hp_tuning'] * 0.745699872) : '-'; 
						$stageData[$i]['kw_tuning_increase'] = $stage['data']['hp_tuning'] > 0 ? round($stage['data']['hp_tuning'] * 0.745699872) - round($stage['data']['hp_ori'] * 0.745699872) : '-';
						$stageData[$i]['kw_tuning_increase_percentage'] = $this->helpers->calculatePercentageIncrease($stage['data']['hp_ori'], $stage['data']['hp_tuning']);
						$stageData[$i]['show_price'] = $this->settings['show_price'];
						$stageData[$i]['price'] = ( (isset($this->settings['fixed_stage_prices'][$data['vehicle']['brand']['type']][$stage['name']])) && (strlen($this->settings['fixed_stage_prices'][$data['vehicle']['brand']['type']][$stage['name']]) > 0) ) ? $this->settings['fixed_stage_prices'][$data['vehicle']['brand']['type']][$stage['name']] : $stage['price'];
					} else {
						$stageDisplayData['in_development'] = true;
					}
				}
				$i++;
			}	
		}
		return $stageData;
	}
}
       
$ctrDisplay = new Ctr_Display();
$ctrHelpers = new Ctr_Helpers();
$ctrDatabase = new Ctr_DB();
$ctrMedia = new CTR_Media();
$ctrDisplay->init();
$ctrDisplay->setNavigationData();
$navigationData = $ctrDisplay->getNavigationData();
$data = $ctrDisplay->handleFront();

if(isset($data['is_ajax'])) {
	$ctrDisplay->handleAjax($data);
	exit;
}


/**
 * DO NOT REMOVE
 * SET VIEW VARIABLES
 */

$PLUGIN_ROOT_URL    = get_site_url() . '/'. $ctrDisplay->getSettings()['start_url'];
$ASSETS_URL         = content_url('uploads/ctr/assets/');
$MEDIA_URL          = get_site_url() . '/'. $ctrDisplay->getSettings()['start_url'] . '/assets/media/';
$LANGUAGE          = $ctrDisplay->getSettings()['display_lang'];
$CUSTOM_TPL_PATH	= wp_upload_dir()['basedir'] . '/ctr/custom_templates/';
